 
/*--------------------------------------------------------------------------+
 | Functions and data common to LUX17 and LUX23                             |
 |                                                                          |
 | LUX17 and LUX23 are programs for controlling respectively the X-10 UX17A |
 | "IR Commander" and the UX23A "VCR Commander II" devices under the Linux  |
 | operating system.                                                        |
 |                                                                          |
 | LUXnn uses Heyu and an X-10 CM17A "Firecracker" RF transmitter to        |
 | configure the UXnnA device for sending IR signals to a TV, VCR, DVD,     |
 | CAB, SAT, and/or CD appliance under computer control.                    |
 |                                                                          |
 |              Copyright 2007 Charles W. Sullivan                          |
 |                     All Rights Reserved                                  |
 |                                                                          |
 | This program is free software; you can redistribute it and/or modify     |
 | it under the terms of the GNU General Public License as published by     |
 | the Free Software Foundation; either version 2 of the License, or        |
 | (at your option) any later version.                                      |
 |                                                                          |
 | This program is distributed in the hope that it will be useful,          |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of           |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            |
 | GNU General Public License for more details.                             |
 |                                                                          |
 | You should have received a copy of the GNU General Public License        |
 | along with this program; if not, write to the Free Software Foundation,  |
 | Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.           |
 +--------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <ctype.h>

#if defined(UX23A_DEVICE)
#include "lux23.h"
#else
#include "lux17.h"
#endif


/* Adjust VOLUME_DELAY for a reasonable step up/down in the */
/* TV Volume when the 'volup' or 'voldn' command is issued. */
#define VOLUME_DELAY    500  /* Milliseconds */

/* Command to test the configuration */
#define CODENUM_CONFIG_COMMAND  CODENUM_POWER

#define HCMD "heyu fflux 3 300"
#define SCMD "heyu sleep"

#define CONFIG_DELAY      4  /* Seconds */
#define INIT_DELAY        4  /* Seconds */
#define POST_DELAY        2  /* Seconds */


struct {
   unsigned int code;     /* Heyu */
   char         *ahpcmd;  /* AHP */
} rf_cmd[64] = {
   {0x8203, "00TV"},
   {0x8207, "01Mute"},
   {0x820b, "02ChannelDown"},
   {0x820f, "03Power"},
   {0x8213, "04ChannelUp"},
   {0x8217, "05Backlight1"},
   {0x821b, "06VCRAux11"},
   {0x821f, "07VolumeDown"},
   {0x8223, "08Rewind"},
   {0x8227, "09Play"},
   {0x822b, "10FastForward"},
   {0x822f, "11VCRAuxA"},
   {0x8233, "12CabAuxB"},
   {0x8237, "13VolumnUp"}, /* sic */
   {0x823b, "14Record"},
   {0x823f, "15Stop"},
   {0x8243, "16Pause"},
   {0x8247, "17Aux8"},
   {0x824b, "18SatAuxC"},
   {0x824f, "19SetupAndShift"},
   {0x8253, "20One"},
   {0x8257, "21Two"},
   {0x825b, "22Three"},
   {0x825f, "23Setup"},
   {0x8263, "24X10AuxA"},
   {0x8267, "25Display"},
   {0x826b, "26Four"},
   {0x826f, "27Five"},
   {0x8273, "28Six"},
   {0x8277, "29MenuToggle"},
   {0x827b, "30MouseAuxB"},
   {0x827f, "31RecallOrLast"},
   {0x8303, "32DVDAuxD"},
   {0x8307, "33SatAuxC"},
   {0x830b, "34CabAuxB"},
   {0x830f, "35VolumeUp"},
   {0x8313, "36AudAuxE"},
   {0x8317, "37VolumeDown"},
   {0x831b, "38Seven"},
   {0x831f, "39Eight"},
   {0x8323, "40Nine"},
   {0x8327, "41Sleep1"},
   {0x832b, "42CDAux4"},
   {0x832f, "43Aux10"},
   {0x8333, "44ABSwitch"},
   {0x8337, "45Zero"},
   {0x833b, "46Enter"},
   {0x833f, "47TVOrVCR"},
   {0x8343, "48Backlight2"},
   {0x8347, "49Up"},
   {0x834b, "50Left"},
   {0x834f, "51Okey"},
   {0x8353, "52Right"},
   {0x8357, "53Shift"},
   {0x835b, "54X10FourOn"},
   {0x835f, "55X10FourOff"},
   {0x8363, "56Menu"},
   {0x8367, "57Down"},
   {0x836b, "58Exit"},
   {0x836f, "59Unknown"},
   {0x8373, "60X10OneOn"},
   {0x8377, "61X10TwoOn"},
   {0x837b, "62X10ThreeOn"},
   {0x837f, "63Release"},
};

int vol_delay(), set_relflag(), reset_relflag(), func_parm();

struct command_st {
   char *cmd;
   char *parm;
   char lbrk;            /* Print line break for "show commands" */
   int  codenum;         /* Index into rf_cmd[] table above */
   int  (*add_parms)();
} command[] = {
   {"tv", "",       1, CODENUM_TV,  NULL},
   {"vcr", "",      0, CODENUM_VCR, NULL},
   {"dvd", "",      0, CODENUM_DVD, NULL},
   {"cab", "",      0, CODENUM_CAB, NULL},
   {"sat", "",      0, CODENUM_SAT, NULL},
   {"cd", "",       0, CODENUM_CD,  NULL},
   {"power", "",    1, CODENUM_POWER, NULL},
   {"chanup", "",   0, CODENUM_CHANUP, NULL},
   {"chandn", "",   0, CODENUM_CHANDN, NULL},
   {"volup", "",    0, CODENUM_VOLUP, vol_delay},
   {"voldn", "",    0, CODENUM_VOLDN, vol_delay},
   {"mute", "",     0, CODENUM_MUTE, NULL},
   {"menu", "",     1, CODENUM_MENU, NULL},
   {"up", "",       0, CODENUM_UP, NULL},
   {"down", "",     0, CODENUM_DOWN, NULL},
   {"right", "",    0, CODENUM_RIGHT, NULL},
   {"left", "",     0, CODENUM_LEFT, NULL},
   {"skipup", "",   0, CODENUM_SKIPUP, NULL},
   {"skipdn", "",   0, CODENUM_SKIPDN, NULL},
   {"play", "",     1, CODENUM_PLAY, NULL},
   {"pause", "",    0, CODENUM_PAUSE, NULL},
   {"stop", "",     0, CODENUM_STOP, NULL},
   {"record", "",   0, CODENUM_RECORD, NULL},
   {"fastfwd", "",  0, CODENUM_FASTFWD, NULL},
   {"rewind", "",   0, CODENUM_REWIND, NULL},
   {"exit", "",     1, CODENUM_EXIT, NULL},
   {"title", "",    0, CODENUM_TITLE, NULL},
   {"subtitle", "", 0, CODENUM_SUBTITLE, NULL},
   {"display", "",  0, CODENUM_DISPLAY, NULL},
   {"return", "",   0, CODENUM_RETURN, NULL},
   {"ok", "",       0, CODENUM_OK, NULL},
   {"av", "",       0, CODENUM_AV, NULL},
   {"<digits>", "", 1, CODENUM_SHOWONLY, NULL},
   {"zero", "",     0, CODENUM_ZERO,  NULL},
   {"one", "",      0, CODENUM_ONE,   NULL},
   {"two", "",      0, CODENUM_TWO,   NULL},
   {"three", "",    0, CODENUM_THREE, NULL},
   {"four", "",     0, CODENUM_FOUR,  NULL},
   {"five", "",     1, CODENUM_FIVE,  NULL},
   {"six", "",      0, CODENUM_SIX,   NULL},
   {"seven", "",    0, CODENUM_SEVEN, NULL},
   {"eight", "",    0, CODENUM_EIGHT, NULL},
   {"nine", "",     0, CODENUM_NINE,  NULL},
   {"enter", "",    0, CODENUM_ENTER, NULL},
   {"release", "",  1, CODENUM_RELEASE, NULL},
   {"setup", "",    0, CODENUM_SETUP, NULL},
   {"relf", "",     0, CODENUM_RELF, set_relflag},
   {"norelf", "",   0, CODENUM_NORELF, reset_relflag},
   {"func", "##",   0, CODENUM_FUNC, func_parm},
};


typedef struct {
   char *name;
   char *codelist;
} manf_t;

manf_t manf_tv[] = {
   {"ABEX","185 + 172 185"},
   {"ACME","003 + 004"},
   {"ADA","011 023 + 030 010"},
   {"ADC","003 006 099 + 000 008 086"},
   {"ADMIRAL","014 186 + 001 173"},
   {"ADVENTURA","187 + 174"},
   {"AIKO","071 + 058"},
   {"AKAI","015 + 002"},
   {"ALLERON","059 + 046"},
   {"AMARK","033 204 + 020 191"},
   {"AMSTRAD","202 + 189"},
   {"AMTRON","066 + 053"},
   {"ANAM_NATIONAL","016 038 055 066 206 + 003 025 042 053 193"},
   {"AOC","017 018 020 022 027 145 169 188 + 004 005 007 009 014 132 156 175"},
   {"ARCHER","033 + 020"},
   {"AUDIOVOX","066 + 053"},
   {"BELCOR","017 + 004"},
   {"BELL_&_HOWELL","013 014 062 + 000 001 049"},
   {"BRADFORD","066 + 053"},
   {"BROKSONIC","149 160 + 136 147"},
   {"BROKWOOD","017 + 004"},
   {"CANDLE","017 021 022 187 + 004 008 009 174"},
   {"CAPEHART","188 + 175"},
   {"CELEBRITY","015 + 002"},
   {"CENTURION","022 + 009"},
   {"CETRONIC","055 + 042"},
   {"CITIZEN","014 017 021 022 055 066 071 118 122 197 190 + 001 004 008 009 042 053 058 105 109 184 177"},
   {"CLAIRTONE","027 + 014"},
   {"CLASSIC","055 + 042"},
   {"CLP","072 + 059"},
   {"COLORTYME","017 022 023 + 004 009 010"},
   {"CONCERTO","017 022 + 004 009"},
   {"CONTEC/CONY","022 024 025 026 027 055 066 + 009 011 012 013 014 042 053"},
   {"COSCO","067 079 080 089 082 073 071 072 084 + 032 050 048 048 055"},
   {"COSMO","067 072 065 082 076 069 083 032 + 087 046 032 083 085 076 076 073 086 065 078"},
   {"CRAIG","055 066 + 042 053"},
   {"CROWN","022 055 066 + 009 042 053"},
   {"CURTIS_MATHES","013 014 017 022 028 044 062 118 122 + 000 001 004 009 015 031 049 105 109"},
   {"CXC","055 066 + 042 053"},
   {"DAEWOO","017 018 022 030 031 032 055 071 095 096 098 110 113 139 140 209 + 004 005 009 017 018 019 042 058 082 083 085 097 100 126 127 196 "},
   {"DAYTRON","017 022 + 004 009"},
   {"DIMENSIA","013 044 + 000 031"},
   {"DUMONT","017 134 + 004 121"},
   {"DYNASTY","055 + 042"},
   {"ELEKTRA","014 + 001"},
   {"ELECTROBAND","015 027 + 002 014"},
   {"ELECTROHOME","016 017 018 022 035 146 + 003 004 005 009 022 133"},
   {"EMERSON","017 022 027 036 037 038 039 040 043 045 046 047 048 049 050 051 052 053 054 055 056 057 058 059 062 066 129 136 148 160 190 192 217 + 004 009 014 023 024 025 026 027 030 032 033 034 035 036 037 038 039 040 041 042 043 044 045 046 049 053 116 123 135 147 177 179 204 "},
   {"ENVISION","017 022 + 004 009"},
   {"FISHER","026 062 063 193 + 013 049 050 180"},
   {"FUJITSU","059 + 046"},
   {"FUNAI","055 059 066 + 042 046 053"},
   {"FUTURETEC","055 066 + 042 053"},
   {"GE","013 016 017 022 035 044 057 065 067 068 100 105 116 120 138 177 180 194 + 000 003 004 009 022 031 044 052 054 055 087 092 103 107 125 164 167 181"},
   {"GIBRALTAR","017 134 + 004 121"},
   {"GOLDSTAR","017 018 022 069 070 146 168 169 185 + 004 005 009 056 057 133 155 156 172"},
   {"GRUNDY","022 059 066 + 009 046 053"},
   {"HALLMARK","017 022 + 004 009"},
   {"HARVARD","066 + 053"},
   {"HITACHI","014 017 022 024 026 072 073 074 100 150 151 152 153 154 155 156 157 158 159 192 + 001 004 009 011 013 059 060 061 087 137 138 139 140 141 142 143 144 145 146 179"},
   {"IMA","066 + 053"},
   {"INFINITY","075 + 062"},
   {"INTEQ","134 + 121"},
   {"JANEIL","187 + 174"},
   {"JBL","075 + 062"},
   {"JCB","015 + 002"},
   {"JC_PENNEY","013 017 018 021 022 035 044 065 067 068 076 100 118 120 122 141 185 194 + 000 004 005 008 009 022 031 052 054 055 063 087 105 107 109 128 172 181"},
   {"JENSEN","017 022 + 004 009"},
   {"JVC","022 024 026 067 068 073 078 080 102 171 172 + 009 011 013 054 055 060 065 067 089 158 159"},
   {"KAWASHO","015 017 022 + 002 004 009"},
   {"KAYPANI","188 + 175"},
   {"KEC","055 + 042"},
   {"KENWOOD","017 022 146 + 004 009 133"},
   {"KLOSS_NOVABEAM","081 082 187 + 068 069 174"},
   {"KTV","022 027 034 055 066 083 129 190 + 009 014 021 042 053 070 116 177"},
   {"LODGENET","013 014 + 000 001"},
   {"LOEWE","075 + 062"},
   {"LOGIK","013 014 + 000 001"},
   {"LUXMAN","017 022 + 004 009"},
   {"LXI","013 017 022 044 062 075 120 122 194 + 000 004 009 031 049 062 107 109 181"},
   {"MAGNAVOX","017 021 022 075 081 082 087 088 089 090 102 146 178 + 004 008 009 062 068 069 074 075 076 077 089 133 165"},
   {"MAJESTIC","013 014 + 000 001"},
   {"MARANTZ","017 022 075 191 + 004 009 062 178"},
   {"MATSUI","075 + 062"},
   {"MEGATRON","017 022 072 + 004 009 059"},
   {"MEI","027 + 014"},
   {"MEMOREX","013 014 017 018 022 059 062 064 148 + 000 001 004 005 009 046 049 051 135"},
   {"MGA","017 018 022 035 059 092 146 193 + 004 005 009 022 046 079 133 180"},
   {"MIDLAND","022 067 068 120 134 185 194 + 009 054 055 107 121 172 181"},
   {"MINUTZ","065 + 052"},
   {"MITSUBISHI","017 018 022 035 059 092 094 102 138 145 146 193 + 004 005 009 022 046 079 081 089 125 132 133 180"},
   {"MONTGOMERY_WARD","013 014 + 000 001"},
   {"MOTOROLA","016 186 + 003 173"},
   {"MTC","017 018 022 027 118 122 + 004 005 009 014 105 109"},
   {"MULTITECH","066 + 053"},
   {"MULTIVISION","097 + 084"},
   {"NAD","017 022 122 198 + 004 009 109 185"},
   {"NEC","016 017 018 022 023 098 102 108 + 003 004 005 009 010 085 089 095"},
   {"NIKEI","055 + 042"},
   {"NIKKO","017 022 071 + 004 009 058"},
   {"NTC","071 + 058"},
   {"ONKING","055 + 042"},
   {"ONWA","055 066 + 042 053"},
   {"OPTIMUS","183 198 + 170 185"},
   {"OPTONICA","108 186 + 095 173"},
   {"ORION","048 134 148 + 035 121 135"},
   {"PANASONIC","016 067 068 075 162 183 184 + 003 054 055 062 149 170 171"},
   {"PHILCO","016 017 018 021 022 075 081 082 087 088 090 146 + 003 004 005 008 009 062 068 069 074 075 077 133"},
   {"PHILIPS","016 017 019 021 022 075 081 082 087 088 089 099 100 102 146 + 003 004 006 008 009 062 068 069 074 075 076 086 087 089 133"},
   {"PILOT","017 022 + 004 009"},
   {"PIONEER","017 022 103 105 192 198 + 004 009 090 092 179 185"},
   {"PORTLAND","017 018 022 071 + 004 005 009 058"},
   {"PRICE_CLUB","118 + 105"},
   {"PRISM","067 068 + 054 055"},
   {"PROSCAN","013 044 120 194 + 000 031 107 181"},
   {"PROTON","017 022 106 188 199 + 004 009 093 175 186"},
   {"PULSAR","134 + 121"},
   {"PULSER","017 + 004"},
   {"QUASAR","016 067 068 083 107 + 003 054 055 070 094"},
   {"RADIO_SHACK/REALISTIC","013 017 022 024 044 054 055 061 062 066 108 168 183 185 + 000 004 009 011 031 041 042 048 049 053 095 155 170 172"},
   {"RCA","013 016 017 018 020 022 024 044 061 091 095 096 109 111 112 114 115 116 120 126 142 146 192 194 200 201 + 000 003 004 005 007 009 011 031 048 078 082 083 096 098 099 101 102 103 107 113 129 133 179 181 187 188"},
   {"RHAPSODY","027 + 014"},
   {"RUNCO","134 + 121"},
   {"SAMPO","017 022 185 188 + 004 009 172 175"},
   {"SAMSUNG","017 018 022 024 028 117 118 119 122 146 185 + 004 005 009 011 015 104 105 106 109 133 172"},
   {"SANSUI","148 149 + 135 136 "},
   {"SANYO","017 026 062 121 193 + 004 013 049 108 180"},
   {"SCOTCH","017 022 + 004 009"},
   {"SCOTT","017 022 037 048 055 059 066 + 004 009 024 035 042 046 053"},
   {"SEARS","013 017 022 026 044 059 062 118 120 122 123 146 193 194 202 + 000 004 009 013 031 046 049 105 107 109 110 133 180 181 189"},
   {"SHARP","017 022 108 124 125 127 135 136 137 186 187 + 004 009 095 111 112 114 122 123 124 173 174"},
   {"SHOGUN","017 + 004"},
   {"SIGNATURE","013 014 036 + 000 001 023"},
   {"SIMPSON","021 + 008"},
   {"SONIC","027 + 014"},
   {"SONY","015 019 + 002 006"},
   {"SOUNDESIGN","017 021 022 055 059 066 + 004 008 009 042 046 053"},
   {"SQUAREVIEW","202 + 189"},
   {"SSS","017 055 066 + 004 042 053"},
   {"STARLITE","066 + 053"},
   {"SUPRE-MACY","187 + 174"},
   {"SUPREME","015 + 002"},
   {"SYLVANIA","017 021 022 075 081 082 087 088 089 090 146 174 + 004 008 009 062 068 069 074 075 076 077 133 161"},
   {"SYMPHONIC","046 066 202 + 033 053 189"},
   {"TANDY","186 + 173"},
   {"TATUNG","016 191 + 003 178"},
   {"TECHNICS","067 068 + 054 055"},
   {"TECHWOOD","017 022 067 068 + 004 009 054 055"},
   {"TEKNIKA","013 014 017 018 021 022 026 055 059 066 071 089 118 122 183 187 +  000 001 004 005 008 009 013 042 046 053 058 076 105 109 170 174"},
   {"TELECAPTION","130 + 117"},
   {"TELERENT","013 014 + 000 001"},
   {"TERA","017 199 + 004 186"},
   {"TMK","017 022 + 004 009"},
   {"TOSHIBA","026 062 102 118 122 130 131 + 013 049 089 105 109 117 118"},
   {"UNIVERSAL","065 100 + 052 087"},
   {"VICTOR","073 + 060"},
   {"VIDTECH","017 018 022 + 004 005 009"},
   {"VIKING","187 + 174"},
   {"WARDS","013 014 017 018 022 037 044 046 059 065 075 081 082 087 088 089 100 108 132 133 146 + 000 001 004 005 009 024 031 033 046 052 062 068 069 074 075 076 087 095 119 120 133"},
   {"WHITE_WESTINGHOUSE","110 113 + 097 100"},
   {"YAMAHA","017 018 022 146 + 004 005 009 133"},
   {"YUPITERU","055 + 042"},
   {"ZENITH","013 014 017 036 051 071 077 134 148 149 166 + 000 001 004 023 038 058 064 121 135 136 153"},
};


manf_t manf_vcr[] = {
   {"ADMIRAL","014 + 206"},
   {"ADVENTURA","039 + 231"},
   {"AIKO","040 + 232"},
   {"AIWA","015 039 057 + 207 231 249"},
   {"AKAI","016 017 018 019 021 042 124 125 + 208 209 210 211 213 234 316 317"},
   {"ALLEGRO","031 035 117 + 223 227 309"},
   {"AMERICAN_HIGH","034 073 + 226 265"},
   {"ASHA","026 + 218"},
   {"AUDIOVOX","027 + 219"},
   {"AUDIO_DYNAMICS","022 136 + 214 328"},
   {"BELL","024 + 216"},
   {"BELL_&_HOWELL","024 + 216"},
   {"BEAUMARK","026 + 218"},
   {"BROKSONIC","025 038 044 077 098 + 217 230 236 269 290"},
   {"CALIX","027 + 219"},
   {"CANDLE","022 026 027 028 029 030 032 + 214 218 219 220 221 222 224"},
   {"CANNON","034 073 127 128 + 226 265 319 320"},
   {"CAPEHART","117 123 + 309 315"},
   {"CARVER","075 + 267"},
   {"CCE","040 074 + 232 266"},
   {"CITIZEN","022 026 027 028 029 030 032 040 + 214 218 219 220 221 222 224 232"},
   {"COLORTYME","022 + 214"},
   {"COLT","074 + 266"},
   {"CRAIG","026 027 036 074 + 218 219 228 266"},
   {"CURTIS_MATHES","013 015 022 026 029 034 037 073 128 + 205 207 214 218 221 226 229 265 320"},
   {"CYBERNEX","026 + 218"},
   {"DAEWOO","028 030 032 038 039 040 041 123 + 220 222 224 230 231 232 233 315"},
   {"DAYTRON","123 + 315"},
   {"DBX","022 136 + 214 328"},
   {"DIMENSIA","013 + 205"},
   {"DYNATECH","015 039 + 207 231"},
   {"ELECTROHOME","027 042 + 219 234"},
   {"ELECTROPHONIC","027 + 219"},
   {"EMERSON","015 019 025 027 028 034 037 038 039 042 043 044 045 046 047 048 049 050 051 052 053 054 056 057 058 060 073 078 118 129 + 207 211 217 219 220 226 229 230 231 234 235 236 237 238 239 240 241 242 243 244 245 246 248 249 250 252 265 270 310 321"},
   {"FISHER","022 024 036 061 062 063 075 + 214 216 228 253 254 255 267"},
   {"FUJI","017 034 073 + 209 226 265"},
   {"FUNAI","015 021 039 + 207 213 231"},
   {"GARRARD","039 + 231"},
   {"GE","013 026 034 066 073 128 133 + 205 218 226 258 265 320 325"},
   {"GOLDSTAR","022 027 067 134 + 214 219 259 326"},
   {"GRADIENTE","039 + 231"},
   {"GO-VIDEO","147 156 + 339 348"},
   {"HARLEY_DAVIDSON","039 + 231"},
   {"HARMAN/KARDON","022 081 + 214 273"},
   {"HARWOOD","074 + 266"},
   {"HEADQUARTER","024 + 216"},
   {"HITACHI","015 068 069 070 120 124 133 135 157 158 + 207 260 261 262 312 316 325 327 349 350"},
   {"HI-Q","036 + 228"},
   {"INSTANT_REPLAY","021 034 073 + 213 226 265"},
   {"INTEQ","027 + 219"},
   {"JCL","034 073 + 226 265"},
   {"JC_PENNEY","022 024 026 027 068 069 073 120 124 128 131 136 + 214 216 218 219 260 261 265 312 316 320 323 328"},
   {"JENSEN","068 124 + 260 316"},
   {"JVC","022 024 035 124 136 + 214 216 227 316 328"},
   {"KENWOOD","022 024 029 124 136 + 214 216 221 316 328"},
   {"KLH","074 144 + 266 336"},
   {"KODAK","027 034 073 + 219 226 265"},
   {"LLOYD","015 039 + 207 231"},
   {"LOGIK","074 + 266"},
   {"LXI","027 + 219"},
   {"MAGNAVOX","034 073 075 076 117 119 128 137 + 226 265 267 268 309 311 320 329"},
   {"MAGNIN","026 + 218"},
   {"MARANTZ","022 024 029 034 073 075 077 127 136 + 214 216 221 226 265 267 269 319 328"},
   {"MARTA","027 + 219"},
   {"MASUSHITA","034 073 + 226 265"},
   {"MEI","034 073 + 226 265"},
   {"MEMOREX","014 015 024 026 027 034 036 039 056 073 117 125 + 206 207 216 218 219 226 228 231 248 265 309 317"},
   {"MGA","042 078 + 234 270"},
   {"MGN_TECHNOLOGY","026 + 218"},
   {"MIDLAND","066 + 258"},
   {"MINOLTA","068 069 120 + 260 261 312"},
   {"MITSUBISHI","042 068 069 078 079 081 082 083 084 086 087 136 + 234 260 261 270 271 273 274 275 276 278 279 328"},
   {"MONTGOMERY_WARD","014 088 + 206 280"},
   {"MOTOROLA","014 034 073 + 206 226 265"},
   {"MTC","015 026 039 + 207 218 231"},
   {"MULTITECH","015 026 039 066 074 + 207 218 231 258 266"},
   {"NEC","022 024 029 037 077 089 090 091 124 136 + 214 216 221 229 269 281 282 283 316 328"},
   {"NIKKO","027 + 219"},
   {"NOBLEX","026 + 218"},
   {"OLYMPUS","034 073 + 226 265"},
   {"OPTIMUS","014 027 122 149 150 + 206 219 314 341 342"},
   {"OPTONICA","088 147 + 280 339"},
   {"PANASONIC","034 073 122 128 138 139 140 + 226 265 314 320 330 331 332"},
   {"PENTAX","029 068 120 133 + 221 260 312 325"},
   {"PENTEX_RESEARCH","022 + 214"},
   {"PHILCO","034 073 075 076 128 + 226 265 267 268 320"},
   {"PHILIPS","034 073 075 088 119 137 + 226 265 267 280 311 329"},
   {"PILOT","027 + 219"},
   {"PIONEER","068 093 094 136 + 260 285 286 328"},
   {"PORTLAND","029 030 032 123 + 221 222 224 315"},
   {"PROSCAN","013 + 205"},
   {"PROTEC","074 + 266"},
   {"PULSAR","117 + 309"},
   {"QUARTER","024 + 216"},
   {"QUARTZ","024 + 216"},
   {"QUASAR","034 073 128 138 + 226 265 320 330"},
   {"RCA","013 016 026 034 068 069 073 095 096 097 098 099 100 101 102 103 104 120 128 133 138 + 205 208 218 226 260 261 265 287 288 289 290 291 292 293 294 295 296 312 320 325 330"},
   {"RADIO_SHACK/REALISTIC","013 014 015 024 026 027 034 036 039 042 062 063 073 088 122 125 128 149 150 151 + 205 206 207 216 218 219 226 228 231 234 254 255 265 280 314 317 320 341 342 343"},
   {"RADIX","027 + 219"},
   {"RANDEX","027 + 219"},
   {"RICOH","141 + 333"},
   {"RUNCO","117 + 309"},
   {"SAMSUNG","018 026 028 046 066 068 125 147 + 210 218 220 238 258 260 317 339"},
   {"SANKYO","014 117 + 206 309"},
   {"SANSUI","105 124 136 + 297 316 328"},
   {"SANYO","024 026 036 + 216 218 228"},
   {"SCOTT","025 028 038 045 051 078 106 129 + 217 220 230 237 243 270 298 321"},
   {"SEARS","024 027 034 036 061 062 063 068 069 073 075 120 131 + 216 219 226 228 253 254 255 260 261 265 267 312 323"},
   {"SHARP","014 015 030 042 088 148 149 + 206 207 222 234 280 340 341"},
   {"SHINTOM","017 068 069 074 141 145 + 209 260 261 266 333 337"},
   {"SHOGUN","026 + 218"},
   {"SIGNATURE","015 + 207"},
   {"SINGER","034 073 074 + 226 265 266"},
   {"SONY","017 112 141 152 153 154 + 209 304 333 344 345 346"},
   {"STS","034 068 073 120 + 226 260 265 312"},
   {"SYLVANIA","015 039 073 075 076 078 128 137 + 207 231 265 267 268 270 320 329"},
   {"SYMPHONIC","015 039 + 207 231"},
   {"TANDY","015 024 + 207 216"},
   {"TASHIKO","027 + 219"},
   {"TATUNG","124 + 316"},
   {"TEAC","015 039 098 124 + 207 231 290 316"},
   {"TECHNICS","034 073 122 + 226 265 314"},
   {"TEKNIKA","015 027 034 039 073 113 142 + 207 219 226 231 265 305 334"},
   {"TOSHIBA","028 062 068 075 078 106 129 + 220 254 260 267 270 298 321"},
   {"TOTEVSION","026 027 + 218 219"},
   {"TMK","026 037 060 + 218 229 252"},
   {"UNITECH","026 + 218"},
   {"VECTOR_RESEARCH","022 028 029 136 + 214 220 221 328"},
   {"VICTOR","136 + 328"},
   {"VIDEOSONIC","026 + 218"},
   {"VIDEO_CONCEPTS","022 028 029 042 136 + 214 220 221 234 328"},
   {"WARDS","014 015 026 027 028 034 036 039 042 068 069 073 074 088 114 120 129 + 206 207 218 219 220 226 228 231 234 260 261 265 266 280 306 312 321"},
   {"WHITE_WESTINGHOUSE","015 021 + 207 213"},
   {"XR-1000","034 039 073 074 + 226 231 265 266"},
   {"YAMAHA","022 024 124 136 + 214 216 316 328"},
   {"ZENITH","017 020 023 027 031 035 098 117 132 136 141 146 + 209 212 215 219 223 227 290 309 324 328 333 338"},
};

manf_t manf_dvd[] = {
   {"AKAI","165 + 357"},
   {"DENON","160 170 + 352 362"},
   {"HITACHI","174 + 366"},
   {"JVC","166 176 + 358 368"},
   {"MAGNAVOX","168 178 + 360 370"},
   {"MARANTZ","158 168 + 350 360"},
   {"MITSUBISHI","162 172 + 354 364"},
   {"PANASONIC","160 161 170 171 + 352 353 362 363"},
   {"PHILIPS","158 168 178 + 350 360 370"},
   {"PIONEER","164 174 + 356 366"},
   {"PROSCAN","163 173 + 355 365"},
   {"RCA","163 173 + 355 365"},
   {"SAMSUNG","177 + 369"},
   {"SHARP","167 + 359"},
   {"SONY","159 169 + 351 361"},
   {"TOSHIBA","158 168 + 350 360"},
   {"YAMAHA","170 + 362"},
   {"ZENITH","158 165 166 175 + 350 357 358 367"},
};

manf_t manf_cab[] = {
   {"ABC","014 015 016 017 018 019 020 + 381 382 383 384 385 386 387"},
   {"ALLEGRO","024 077 078 + 391 444 445"},
   {"AMERICAST","080 + 447"},
   {"ANTRONIX","021 022 + 388 389"},
   {"ARCHER","021 022 023 024 + 388 389 390 391"},
   {"CABLETENNA","021 + 388"},
   {"CABLEVIEW","021 + 388"},
   {"CENTURY","024 + 391"},
   {"CITIZEN","024 + 391"},
   {"COLOUR_VOICE","025 026 + 392 393"},
   {"COMTRONICS","027 028 + 394 395"},
   {"CONTEC/CONY","029 + 396"},
   {"EASTERN","030 + 397"},
   {"GARRARD","024 + 391"},
   {"GC_ELECTRONICS","022 + 389"},
   {"GE","102 + 469"},
   {"GEMINI","031 032 + 398 399"},
   {"GENERAL_INSTRUMENT","016 062 064 066 067 071 081 082 083 084 102 + 383 429 431 433 434 438 448 449 450 451 469"},
   {"HAMLIN","033 034 035 + 400 401 402"},
   {"HITACHI","016 + 383"},
   {"HYTEX","015 + 382"},
   {"JASCO","024 + 391"},
   {"JERROLD","014 016 017 018 020 031 037 062 063 079 081 082 + 381 383 384 385 387 398 404 429 430 446 448 449"},
   {"MAGNAVOX","038 + 405"},
   {"MEMOREX","039 + 406"},
   {"MOVIE_TIME","015 040 + 382 407"},
   {"NSC","015 040 + 382 407"},
   {"OAK","015 029 042 + 382 396 409"},
   {"PANASONIC","013 061 + 380 428"},
   {"PARAGON","039 + 406"},
   {"PHILIPS","024 025 026 032 038 043 044 045 + 391 392 393 399 405 410 411 412"},
   {"PIONEER","046 047 051 + 413 414 418"},
   {"PULSAR","039 + 406"},
   {"RADIO_SHACK","058 + 425"},
   {"RCA","013 060 + 380 427"},
   {"REALISTIC","022 + 389"},
   {"REGAL","035 + 402"},
   {"REGENCY","030 + 397"},
   {"REMBRANDT","016 + 383"},
   {"RUNCO","039 + 406"},
   {"SAMSUNG","027 047 + 394 414"},
   {"SCIENTIFIC_ATLANTA","019 049 050 051 + 386 416 417 418"},
   {"SIGNAL","027 031 + 394 398"},
   {"SIGNATURE","016 + 383"},
   {"SL_MARX","027 + 394"},
   {"SPRUCER","013 + 380"},
   {"STANDARD_COMPONENTS","052 + 419"},
   {"STARCOM","014 020 031 + 381 387 398"},
   {"STARGATE","027 031 + 394 398"},
   {"STARQUEST","031 + 398"},
   {"STARSIGHT","073 + 440"},
   {"STARSIGHT_CABLE","060 + 427"},
   {"STAR_TRAK","069 + 436"},
   {"TANDY","053 + 420"},
   {"TELEVIEW","027 + 394"},
   {"TEXSCAN","052 + 419"},
   {"TOCOM","017 054 066 + 384 421 433"},
   {"TOSHIBA","039 + 406"},
   {"TUSA","031 + 398"},
   {"TV86","040 + 407"},
   {"UNIKA","021 022 024 + 388 389 391"},
   {"UNITED_ARTISTS","015 + 382"},
   {"UNITED_CABLE","014 + 381"},
   {"UNIVERSAL","021 022 023 024 055 + 388 389 390 391 422"},
   {"VIDEOTECH","103 + 470"},
   {"VIDEOWAY","057 + 424"},
   {"VIEWSTAR","028 038 040 053 + 395 405 407 420"},
   {"ZENITH","024 039 041 056 070 072 073 074 075 076 077 078 080 + 391 406 408 423 437 439 440 441 442 443 444 445 447"},
};

manf_t manf_sat[] = {
   {"AMPLICA","028 + 495"},
   {"BIRDVIEW","029 + 496"},
   {"CHANNEL_MASTER","030 031 032 + 497 498 499"},
   {"CHAPPARAL","013 014 033 + 480 481 500"},
   {"CHEYENNE","014 + 481"},
   {"DRAKE","015 027 038 039 040 + 482 494 505 506 507"},
   {"DISH_NETWORK","071 081 + 538 548"},
   {"DX_ANTENNA","021 035 + 488 502"},
   {"ECHOSPHERE","071 081 + 538 548"},
   {"ECHOSTAR","041 042 043 071 081 + 508 509 510 538 548"},
   {"FUJITSU","044 045 + 511 512"},
   {"GE","075 089 + 542 556"},
   {"GENERAL_INSTRUMENTS","017 018 019 046 080 095 096 097 098 + 484 485 486 513 547 562 563 564 565"},
   {"HITACHI_DSS","084 085 094 + 551 552 561"},
   {"HUGHES_DSS","075 082 083 085 086 091 + 542 549 550 552 553 558"},
   {"JANEIL","049 + 516"},
   {"JERROLD","050 + 517"},
   {"KENWOOD","051 + 518"},
   {"LUXOR","052 + 519"},
   {"MAGNAVOX_DSS","082 092 + 549 559"},
   {"MACOM_STATE","053 + 520"},
   {"NORSTAT","054 055 + 521 522"},
   {"PANASONIC_DSS","069 079 + 536 546"},
   {"PRIMESTAR","070 080 + 537 547"},
   {"PHILIPS","092 + 559"},
   {"PROSCAN","075 089 + 542 556"},
   {"RCA_DSS","065 075 079 080 089 090 + 532 542 546 547 556 557"},
   {"REALISTIC","020 056 057 + 487 523 524"},
   {"SCIENTIFIC_ATLANTA","060 + 527"},
   {"SIERRA_I,_II,_III","014 + 481"},
   {"SONY_DSS","066 076 + 533 543"},
   {"STAR_SIGHT","073 076 086 + 540 543 553"},
   {"STAR_TRAK","061 + 528"},
   {"STS","021 022 023 024 + 488 489 490 491"},
   {"SUPER_GUIDE","062 + 529"},
   {"TOSHIBA","025 + 492"},
   {"TOSHIBA_DSS","067 068 074 077 078 084 + 534 535 541 544 545 551"},
   {"UNIDEN","026 064 + 493 531"},
   {"UNIDEN_DSS","086 + 553"},
   {"VIDEO_CIPHER_II","027 + 494"},
   {"ZENITH","015 016 027 040 077 078 087 088 099 + 482 483 494 507 544 545 554 555 566"},
};

manf_t manf_cd[] = {
   {"ADC","013 + 580"},
   {"ADCOM","014 067 + 581 634"},
   {"AIWA","015 016 + 582 583"},
   {"AKAI","068 090 183 + 635 657 750"},
   {"AUDIO_ALCHEMY","017 + 584"},
   {"AUDIO_TECHNICA","018 + 585"},
   {"BSR","017 + 584"},
   {"CALIFORNIA_AUDIO_LABS","069 + 636"},
   {"CAPETRONIC","022 + 589"},
   {"CARRERA","017 + 584"},
   {"CARVER","019 020 021 178 + 586 587 588 745"},
   {"CASIO","130 + 697"},
   {"CITIZEN","179 + 746"},
   {"CURTIS_MATHES","130 + 697"},
   {"DENON","023 + 590"},
   {"EMERSON","014 025 026 + 581 592 593"},
   {"FISHER","020 028 029 030 071 + 587 595 596 597 638"},
   {"GE","078 + 645"},
   {"GENEXXA","025 026 031 072 + 592 593 598 639"},
   {"GOLDSTAR","073 + 640"},
   {"HARMAN/KARDON","033 044 + 600 611"},
   {"HITACHI","031 + 598"},
   {"INKEL","034 + 601"},
   {"JVC","035 166 + 602 733"},
   {"JC_PENNEY","043 130 + 610 697"},
   {"KENWOOD","036 037 038 080 081 + 603 604 605 647 648"},
   {"KRELL","019 + 586"},
   {"KYOCERA","013 + 580"},
   {"LUXMAN","039 040 082 091 + 606 607 649 658"},
   {"LXI","025 130 + 592 697"},
   {"MAGNAVOX","019 084 153 + 586 651 720"},
   {"MARANTZ","019 041 042 085 + 586 608 609 652"},
   {"MCS","043 130 + 610 697"},
   {"MGA","044 + 611"},
   {"MISSION","019 + 586"},
   {"MITSUBISHI","044 045 + 611 612"},
   {"NAD","074 092 182 + 641 659 749"},
   {"NAKAMICHI","046 047 184 + 613 614 751"},
   {"NEC","043 + 610"},
   {"NIKKO","018 026 029 + 585 593 596"},
   {"NSM","019 + 586"},
   {"ONKYO","048 049 075 164 + 615 616 642 731"},
   {"OPTIMUS","017 020 021 025 031 037 050 051 052 053 097 100 104 + 584 587 588 592 598 604 617 618 619 620 664 667 671"},
   {"PANASONIC","069 086 093 + 636 653 660"},
   {"PHILIPS","019 084 178 + 586 651 745"},
   {"PIONEER","031 054 055 072 + 598 621 622 639"},
   {"PROTON","019 + 586"},
   {"QUASAR","069 + 636"},
   {"RCA","014 020 025 056 076 148 + 581 587 592 623 643 715"},
   {"RCA_DVD","096 + 663 "},
   {"REALISTIC","014 020 022 025 026 031 042 050 052 + 581 587 589 592 593 598 609 617 619"},
   {"ROTEL","019 + 586"},
   {"RS_ORIGINALS","021 059 + 588 626"},
   {"SAE","019 + 586"},
   {"SAMSUNG","094 + 661"},
   {"SANSUI","019 025 057 076 087 + 586 592 624 643 654"},
   {"SANYO","020 058 071 088 095 + 587 625 638 655 662"},
   {"SCOTT","014 025 026 + 581 592 593"},
   {"SEARS","025 037 042 130 + 592 604 609 697"},
   {"SHARP","032 037 042 + 599 604 609"},
   {"SHERWOOD","034 042 051 059 + 601 609 618 626"},
   {"SHURE","043 + 610"},
   {"SONY","060 061 145 147 153 + 627 628 712 714 720"},
   {"STS","013 + 580"},
   {"SYLVANIA","019 + 586"},
   {"SYMPHONIC","062 + 629"},
   {"TEAC","029 042 052 062 063 064 + 596 609 619 629 630 631"},
   {"TECHNICS","069 086 089 173 + 636 653 656 740"},
   {"THETA_DIGITAL","084 + 651"},
   {"TOSHIBA","074 + 641"},
   {"VECTOR_RESEARCH","017 + 584"},
   {"VICTOR","035 + 602"},
   {"YAMAHA","018 065 066 077 + 585 632 633 644"},
   {"ZENITH","029 083 096 112 137 + 596 650 663 679 704"},
};


/*--------------------------------------------------------------------+
 | Device type mask bits used to select configuration codes.          |
 | DEVMASK1_ are used for the UX17A. Both DEVMASK1_ and DEVMASK2_ are |
 | used for the UX23A.                                                |
 +--------------------------------------------------------------------*/

#define DEVMASK1_TV       0x001
#define DEVMASK1_VCR      0x002
#define DEVMASK1_DVD      0x004
#define DEVMASK1_CAB      0x008
#define DEVMASK1_SAT      0x010
#define DEVMASK1_CD       0x020

#define DEVMASK2_TV       0x040
#define DEVMASK2_VCR      0x080
#define DEVMASK2_DVD      0x100
#define DEVMASK2_CAB      0x200
#define DEVMASK2_SAT      0x400
#define DEVMASK2_CD       0x800

struct devlist_st {
   char    *devlabel;
   char    *caplabel;
   manf_t  *manf;
   int     size;
   int     codenum;
   int     oldmask;
   int     newmask;
} devlist[6] = {
   {"tv",  "TV",  manf_tv,  (sizeof(manf_tv)/sizeof(manf_t)),  CODENUM_TV,  DEVMASK1_TV,  DEVMASK2_TV },
   {"vcr", "VCR", manf_vcr, (sizeof(manf_vcr)/sizeof(manf_t)), CODENUM_VCR, DEVMASK1_VCR, DEVMASK2_VCR},
   {"dvd", "DVD", manf_dvd, (sizeof(manf_dvd)/sizeof(manf_t)), CODENUM_DVD, DEVMASK1_DVD, DEVMASK2_DVD},
   {"cab", "CAB", manf_cab, (sizeof(manf_cab)/sizeof(manf_t)), CODENUM_CAB, DEVMASK1_CAB, DEVMASK2_CAB},
   {"sat", "SAT", manf_sat, (sizeof(manf_sat)/sizeof(manf_t)), CODENUM_SAT, DEVMASK1_SAT, DEVMASK2_SAT},
   {"cd",  "CD",  manf_cd,  (sizeof(manf_cd)/sizeof(manf_t)),  CODENUM_CD,  DEVMASK1_CD,  DEVMASK2_CD },
};

/* Global variables */
int  luxmask = 0;
int  ux_type = 0;
int  config_start_flag = 0;
int  release_flag = 1;
int  last_codenum = -1;
char cmdline[1024];


#ifndef max
#define max( a, b )  ((a) > (b) ? (a) : (b))
#endif

int exec_cmdline ( void );

/*--------------------------------------------------------------------+
 | For debugging                                                      |
 +--------------------------------------------------------------------*/
void tp ( int point )
{
   printf("testpoint %d\n", point);
   fflush(stdout);
   return ;
}

/*--------------------------------------------------------------------+
 | Convert string to lower case.                                      |
 +--------------------------------------------------------------------*/
char *strlower ( char *string )
{
   char *sp = string ;

   while ( *sp ) {
      *sp = tolower((int)(*sp));
      sp++ ;
   }
   return string;
}

/*--------------------------------------------------------------------+
 | Convert string to upper case.                                      |
 +--------------------------------------------------------------------*/
char *strupper ( char *string )
{
   char *sp = string ;

   while ( *sp ) {
      *sp = toupper((int)(*sp));
      sp++ ;
   }

   return string;
}

/*--------------------------------------------------------------------+
 | String with current system time as hh:mm:ss for config display.    |
 +--------------------------------------------------------------------*/
char *timestring ( void )
{
   time_t       now;
   struct tm    *tmp;
   static char  buffer[32];

   now = time(NULL);
   tmp = localtime(&now);

   sprintf(buffer, "%02d:%02d:%02d", tmp->tm_hour, tmp->tm_min, tmp->tm_sec);
   return buffer;
}

/*--------------------------------------------------------------------+
 | Break up the string 'str' into tokens delimited by characters in   |
 | 'delim' and create the list 'tokv' of pointers to these tokens.    |
 | The original string is modified.  Free 'tokv' after use.           |
 +--------------------------------------------------------------------*/
int tokenize ( char *str, char *delim, int *tokc, char ***tokv )
{
   char *sp, *tp;

   *tokc = 0;
   sp = str;
   while ( *sp != '\0' ) {
      /* Bypass leading delimiters */
      while ( *sp != '\0' && strchr(delim, *sp) != NULL )
         sp++;
      if ( *sp == '\0' )
         return 0;
      tp = sp;
      /* Advance to the next delimiter */
      while ( *sp != '\0' && strchr(delim, *sp) == NULL )
         sp++;
      /* Terminate the token if not already at end of string */
      if ( *sp != '\0' )
         *sp++ = '\0';
      /* Allocate space for the pointer */
      if ( *tokc == 0 )
         *tokv = calloc(1, sizeof(char *));
      else
         *tokv = realloc(*tokv, (*tokc + 1) * sizeof(char *));
      (*tokv)[(*tokc)++] = tp;
   }

   return 0;
}

/*--------------------------------------------------------------------+
 | Create ux_type global and device mask for UXnnA                    |
 +--------------------------------------------------------------------*/
void create_luxmask ( void )
{
   int type;

   ux_type = (strcmp(UX_TYPE, "UX17A") == 0) ? 0 : 1;

   luxmask = 0;
   for ( type = 0; type < 6; type++ ) {
      luxmask |= devlist[type].oldmask;
      if ( ux_type )
         luxmask |= devlist[type].newmask;
   }

   return;
}
   

/*--------------------------------------------------------------------+
 | Convert integer digit 0-9 to code number for "zero", "one", "two", |
 | ... ,"nine".                                                       |
 +--------------------------------------------------------------------*/
int digit2codenum( int digit )
{
   static int table[10] = {
      CODENUM_ZERO, CODENUM_ONE, CODENUM_TWO, CODENUM_THREE, CODENUM_FOUR,
      CODENUM_FIVE, CODENUM_SIX, CODENUM_SEVEN, CODENUM_EIGHT, CODENUM_NINE
   }; 

   if ( digit < 0 || digit > 9 )
      return -1;

   return table[digit];
}

/*--------------------------------------------------------------------+
 | Identify the type (0-5) of device - TV, VCR, DVD, etc.             |
 +--------------------------------------------------------------------*/
int identify_devtype ( char *label )
{
   int type;

   for ( type = 0; type < (sizeof(devlist)/sizeof(struct devlist_st)); type++ ) {
      if ( strcmp(label, devlist[type].devlabel) == 0 )
         return type;
   }
   return -1;
}

/*--------------------------------------------------------------------+
 | Search the list of manufacturers for the manf of a given device    |
 | type - TV, VCR, DVD, CAB, SAT, or CD.                              |
 +--------------------------------------------------------------------*/
int find_manf( int type, char *search, char **name, char **codelist )
{
   manf_t *manf;
   int  j, slen, size;

   manf = devlist[type].manf;
   size = devlist[type].size;

   slen = strlen(search);

   strupper(search);

   for ( j = 0; j < size; j++ ) {
      if ( strncmp(search, manf[j].name, slen) == 0 )
         break;
   }
 
   if ( j >= size ) {
      fprintf(stderr, "%s Manufacturer name '%s' not found.\n", devlist[type].caplabel, search);
      return 1;
   }
   else if ( !(strlen(manf[j].name) == slen) && j < (size - 1) &&
              strncmp(search, manf[j+1].name, slen) == 0 ) {
      fprintf(stderr, "%s Manufacturer name '%s' ambiguous - enter more characters.\n",
         devlist[type].caplabel, search);
      return 2;
   }
   else {
      *name = manf[j].name;
      *codelist = manf[j].codelist;
   }

   return 0;
}

/*--------------------------------------------------------------------+
 | Display the manufacturers for devices of the specified type - TV,  |
 | VCR, DVD, CAB, SAT, or CD.                                         |
 +--------------------------------------------------------------------*/
int display_manf ( int type, int pagewidth )
{

   manf_t *manf;
   int    j, k, m, size, collength, totwidth, ncol;
   int    maxwidth[5];

   manf = devlist[type].manf;
   size = devlist[type].size;

   /* Analyze names to determine required columns and widths */
   for ( ncol = 5; ncol > 1; ncol-- ) {
      collength = (size + ncol - 1)/ncol;
      for ( k = 0; k < ncol; k++ )
         maxwidth[k] = 0;
      for ( j = 0; j < collength; j++ ) {
         for ( k = 0; k < ncol; k++ ) {
            m = j + k * collength;
            if ( m >= size ) {
               break;
            }
            maxwidth[k] = max(maxwidth[k], strlen(manf[m].name) + 2);
         }
      }
      totwidth = 0;
      for ( k = 0; k < ncol; k++ )
         totwidth += maxwidth[k];
      if ( totwidth < (pagewidth - 1) )
         break;
   }

   /* Now display the names in columns */
   printf("%s Manufacturers:\n-----------------\n", devlist[type].caplabel);
   collength = (size + ncol - 1)/ncol;
   for ( j = 0; j < collength; j++ ) {
      for ( k = 0; k < ncol; k++ ) {
         m = j + k * collength;
         if ( m >= size ) {
            break;
         }
         printf("%-*s", maxwidth[k], manf[m].name);
      }
      printf("\n");
   }
   return 0;
}

/*--------------------------------------------------------------------+
 | Fill in the array with mask values indicating whether a given      |
 | config code has been identified as being for TV, VCR, etc.         |
 +--------------------------------------------------------------------*/
int build_mask_table ( int *devmask, int size )
{
   manf_t       *manf;
   int          j, k, type, tokc, index;
   char         **tokv;
   char         *codelist;
   int          mask, oldmask, newmask;
   int          count = 0;
   
   for ( j = 0; j < size; j++ )
      devmask[j] = 0;

   for ( type = 0; type < 6; type++ ) {
      manf = devlist[type].manf;
      oldmask = devlist[type].oldmask;
      newmask = devlist[type].newmask;
      
      for ( j = 0; j < devlist[type].size; j++ ) {
         codelist = strdup(manf[j].codelist);
         mask = oldmask;
         tokenize(codelist, " ", &tokc, &tokv);
         for ( k = 0; k < tokc; k++ ) {
            if ( strcmp(tokv[k], "+") == 0 ) {
               mask = newmask;
               continue;
            }
            index = (int)strtol(tokv[k], NULL, 10);
            devmask[index] |= mask & luxmask;
         }
         free(tokv);
         free(codelist);
      }
   }

   for ( j = 0; j < size; j++ ) {
      if ( devmask[j] )
         count++;
   }

   return count;
}

/*--------------------------------------------------------------------+
 | Create the command line to execute for setting up a device with a  |
 | specified code determined by its index into a code table.          |
 +--------------------------------------------------------------------*/
int create_config_cmdline ( int type, int index )
{
   int  codenum, code0, code1, code2, release;

   codenum = devlist[type].codenum;
   release = rf_cmd[CODENUM_RELEASE].code;

   code0 = digit2codenum(index / 100);
   code1 = digit2codenum((index % 100) / 10);
   code2 = digit2codenum(index % 10);
   
   if ( config_start_flag )
      sprintf(cmdline, "%s %x ", HCMD, rf_cmd[codenum].code);
   else
      sprintf(cmdline, "%s ", HCMD);
   config_start_flag = 0;

   sprintf(cmdline + strlen(cmdline),
      "%x; %s %.3f; %s %x %x %x %x %x %x %x %x %x %x %x",
      rf_cmd[CODENUM_SETUP].code,
      SCMD, (float)CONFIG_DELAY,
      HCMD, release,
      rf_cmd[codenum].code, release,
      rf_cmd[code0].code, release,
      rf_cmd[code1].code, release,
      rf_cmd[code2].code, release,
      rf_cmd[CODENUM_CONFIG_COMMAND].code, release);

   return 0;
}

      
int ux_config( int type, int ncodes, char *codelist[] )
{
   int  j, index, count = 0, retcode;
   char *sp, *devlabel;
   int  mask;
   int  devmask[1000];

   devlabel = devlist[type].caplabel;
   mask = devlist[type].oldmask | devlist[type].newmask;
   mask &= luxmask;

   if ( strcmp(codelist[0], "all_manf") == 0 ) {
      build_mask_table(devmask, 1000);
      for ( index = 0; index < 1000; index++ ) {
         if ( devmask[index] & mask )
            count++;
      }
      printf("Testing %d codes.\n", count);
      for ( index = 0; index < 1000; index++ ) {
         if ( devmask[index] & mask ) {
            create_config_cmdline(type, index);
            printf("%s code %03d  ", devlabel, index);
            fflush(stdout);
            retcode = exec_cmdline();
	    if ( retcode != 0 ) {
               printf("\n");
	       return 0;
            }
            printf("[%s]\n", timestring());
            sleep(POST_DELAY);
         }
      }
      return 0;
   }
   else if ( strcmp(codelist[0], "all_associated") == 0 ) {
      build_mask_table(devmask, 1000);
      mask = luxmask;
      for ( index = 0; index < 1000; index++ ) {
         if ( devmask[index] & mask )
            count++;
      }
      printf("Testing %d codes.\n", count);
      for ( index = 0; index < 1000; index++ ) {
         if ( devmask[index] & mask ) {
            create_config_cmdline(type, index);
            printf("%s code %03d  ", devlabel, index);
            fflush(stdout);
            retcode = exec_cmdline();
	    if ( retcode != 0 ) {
               printf("\n");
	       return 0;
            }
            printf("[%s]\n", timestring());
            sleep(POST_DELAY);
         }
      }
      return 0;
   }
   else if ( strcmp(codelist[0], "all_unassociated") == 0 ) {
      build_mask_table(devmask, 1000);
      mask = luxmask;
      for ( index = 0; index < 1000; index++ ) {
         if ( devmask[index] & mask )
            count++;
      }
      printf("Testing %d codes.\n", 1000 - count);
      for ( index = 0; index < 1000; index++ ) {
         if ( (devmask[index] & mask) == 0 ) {
            create_config_cmdline(type, index);
            printf("%s code %03d  ", devlabel, index);
            fflush(stdout);
            retcode = exec_cmdline();
	    if ( retcode != 0 ) {
               printf("\n");
	       return 0;
            }
            printf("[%s]\n", timestring());
            sleep(POST_DELAY);
         }
      }
      return 0;
   }
   else if ( strcmp(codelist[0], "all_codes") == 0 ) {
      printf("Testing 1000 codes.\n");
      for ( index = 0; index < 1000; index++ ) {
         create_config_cmdline(type, index);
         printf("%s code %03d  ", devlabel, index);
         fflush(stdout);
         retcode = exec_cmdline();
	 if ( retcode != 0 ) {
            printf("\n");
	    return 0;
         }
         printf("[%s]\n", timestring());
         sleep(POST_DELAY);
      }
      return 0;
   }

   for ( j = 0; j < ncodes; j++ ) {
      if ( strcmp(codelist[j], "+") == 0 ) {
         if ( ux_type )
            continue;
	 else
	    break;
      }
      count++;
   }
   if ( count > 1 )
      printf("Testing %d codes.\n", count);
   
   for ( j = 0; j < ncodes; j++ ) {
      if ( strcmp(codelist[j], "+") == 0 ) {
         if ( ux_type )
            continue;
	 else
	    break;
      }
      index = (int)strtol(codelist[j], &sp, 10);
      if ( !strchr(" \t\n\r", *sp) ) {
         fprintf(stderr, "Invalid code '%s' - must be numeric 0-999\n", codelist[j]);
         exit(1);
      }
      create_config_cmdline(type, index);
      printf("%s code %03d  ", devlabel, index);
      fflush(stdout);
      retcode = exec_cmdline();
      if ( retcode != 0 ) {
         printf("\n");
         return 0;
      }
      printf("[%s]\n", timestring());
      if ( j < (ncodes - 1) )
         sleep(POST_DELAY);
   }

   return 0;
}

      
/*--------------------------------------------------------------------+
 | Repeatedly transmit the RF code for initializing the UXnnA device. |
 +--------------------------------------------------------------------*/
int initialize ( void )
{
   void init_instruction ( void );

   int  j, retcode;

   init_instruction();

   sprintf(cmdline, "%s %x", HCMD, rf_cmd[CODENUM_INIT].code);

   for ( j = 0; j < 15; j++ ) {
      retcode = exec_cmdline();
      printf("Sent init signal %2d\n", j + 1);
      if ( retcode != 0 ) {
         break;
      }
      sleep(INIT_DELAY);
   }

   return 0;
}

/*--------------------------------------------------------------------+
 | Create the command line for a specific function, e.g., "power",    |
 | "chanup", etc.                                                     |
 +--------------------------------------------------------------------*/
int create_cmdline( int argc, char *argv[] ) 
{
   int j, k, codenum, numeric;
   extern int release_flag;
   char *cmd;

   strcpy(cmdline, HCMD);

   for ( k = 1; k < argc; k++ ) {
      cmd = argv[k];

      strlower(cmd);

      /* Check for a pure numeric */
      numeric = 1;
      for ( j = 0; j < strlen(cmd); j++ ) {
         if ( !isdigit((int)(cmd[j])) ) {
            numeric = 0;
            break;
         }
      }
      if ( numeric ) {
         for ( j = 0; j < strlen(cmd); j++ ) {
            codenum = digit2codenum(cmd[j] - '0');
            sprintf(cmdline + strlen(cmdline), " %x", rf_cmd[codenum].code);
            last_codenum = codenum;
            if ( release_flag || cmd[j + 1] == cmd[j] ) {
               sprintf(cmdline + strlen(cmdline), " %x", rf_cmd[CODENUM_RELEASE].code);
               last_codenum = CODENUM_RELEASE;
            }
         }
         continue;
      }


      for ( j = 0; j < (sizeof(command)/sizeof(struct command_st)); j++ ) {
         if ( strcmp(command[j].cmd, cmd) == 0 ) {
            codenum = command[j].codenum;
            if ( codenum >= 0 ) {
               sprintf(cmdline + strlen(cmdline), " %x", rf_cmd[codenum].code);
               last_codenum = codenum;
            }            
            if ( command[j].add_parms != NULL ) {
               if ( command[j].add_parms(codenum, argc, argv, &k) != 0 )
                  return 1;
               break;
            }
            if ( release_flag && codenum != CODENUM_RELEASE) {
               sprintf(cmdline + strlen(cmdline), " %x", rf_cmd[CODENUM_RELEASE].code);
               last_codenum = CODENUM_RELEASE;
            }
            break;
         }
      }
      if ( j >= (sizeof(command)/sizeof(struct command_st)) ) {
         fprintf(stderr, "Invalid command '%s'\n", cmd);
         return 1;
      }
   }
   if ( last_codenum != CODENUM_RELEASE && last_codenum != CODENUM_NORELF) {
      sprintf(cmdline + strlen(cmdline), " %x", rf_cmd[CODENUM_RELEASE].code);
      last_codenum = CODENUM_RELEASE;
   }
   return 0;
}

/*--------------------------------------------------------------------+
 | Insert a delay between "volup"/"voldn" and "release" codes to      |
 | provide a stup up/down in volume.                                  |
 +--------------------------------------------------------------------*/
int vol_delay ( int codenum, int argc, char *argv[], int *argnum ) 
{
   sprintf(cmdline + strlen(cmdline), ";%s %.3f;%s %x",
     SCMD, ((float)VOLUME_DELAY)/1000., HCMD, rf_cmd[CODENUM_RELEASE].code);
   last_codenum = CODENUM_RELEASE;
   return 0;
}

/*--------------------------------------------------------------------+
 | Retrieve the function code number 0-63 for the arbitrary "func"    |
 | command from the user's input.                                     |
 +--------------------------------------------------------------------*/
int func_parm ( int codenum, int argc, char *argv[], int *argnum )
{
   int  func;
   char *sp;

   if ( *argnum >= argc )
      return 1;

   func = (int)strtol(argv[*argnum + 1], &sp, 10);
   if ( !strchr(" \t\n\r", *sp) || func < 0 || func > 63 ) {
      fprintf(stderr, "Invalid func code '%s'\n", argv[*argnum + 1]);
      return 1;
   }

   sprintf(cmdline + strlen(cmdline), " %x", rf_cmd[func].code);
   last_codenum = func;

   if ( release_flag ) {
      sprintf(cmdline + strlen(cmdline), " %x", rf_cmd[CODENUM_RELEASE].code);
      last_codenum = CODENUM_RELEASE;
   }
   
   (*argnum)++;

   return 0;
}

/*--------------------------------------------------------------------+
 | Set the global flag directing that a "release" code should follow  |
 | each subsequent function code.                                     |
 +--------------------------------------------------------------------*/
int set_relflag ( int codenum, int argc, char *argv[], int *argnum )
{
   release_flag = 1;
   last_codenum = CODENUM_RELF;
   return 0;
}

/*--------------------------------------------------------------------+
 | Direct that the "release" code normally following each subsequent  |
 | function code should be omitted.                                   |
 +--------------------------------------------------------------------*/
int reset_relflag ( int codenum, int argc, char *argv[], int *argnum )
{
   release_flag = 0;
   last_codenum = CODENUM_NORELF;
   return 0;
}


int ux_show ( int argc, char *argv[] ) 
{
   int  j, type, codenum, found;
   char *codelist, *dupelist, *manf_name;
   int  tokc;
   char **tokv = NULL;
   
   char *msg00 = "Device type '%s' not recognized - must be TV, VCR, DVD, CAB, SAT, or CD\n";
   char *msg01 = "manf <device>  List manufacturers of <device>";
   char *msg02 = "codes <device> <manf_name>  Codes applicable to <device> by <manf_name>";
   char *msg03 = "commands";
   char *msg04 = "functions";

   if ( argc < 3 ) {
      fprintf(stderr, "show options are:\n   %s\n   %s\n   %s\n   %s\n", msg01, msg02, msg03, msg04);
      return 1;
   }

   if ( strcmp(argv[2], "manf") == 0 ) {
      if ( argc < 4 ) {
         fprintf(stderr, "<device> parameter (TV, VCR, DVD, CAB, SAT, or CD) required.\n");
         return 1;
      }
      type = identify_devtype(argv[3]);
      if ( type < 0 ) {
         fprintf(stderr, msg00, strupper(argv[3]));
         return 1;
      }
      display_manf(type, 80);
      return 0;
   }
   else if ( strcmp(argv[2], "codes") == 0 ) {
      if ( argc < 5 ) {
         fprintf(stderr, "<device> and <manufacturer> parameters required.\n");
         return 1;
      }
      type = identify_devtype(argv[3]);
      if ( type < 0 ) {
         fprintf(stderr, msg00, strupper(argv[3]));
         return 1;
      }
      if ( find_manf(type, argv[4], &manf_name, &codelist) != 0 ) 
         return 1;

      dupelist = strdup(codelist);
      tokenize(dupelist, " ", &tokc, &tokv);
      printf("%s %s codes:", manf_name, devlist[type].caplabel);
      for ( j = 0; j < tokc; j++ ) {
         if ( strcmp(tokv[j], "+") == 0 ) {
            if ( ux_type == 0 )
               break;
         }
         printf(" %s", tokv[j]);
      }
      printf("\n");
      free(tokv);
      free(dupelist);
      return 0;
   }
   else if ( strcmp(argv[2], "commands") == 0 ) {
      for ( j = 0; j < (sizeof(command)/sizeof(struct command_st)); j++ ) {
         if ( command[j].lbrk )
            printf("\n ");
         if ( command[j].codenum == CODENUM_UNDEF )
            continue;
         if ( *command[j].parm != '\0' )
            printf(" '%s %s'", command[j].cmd, command[j].parm);
         else            
            printf(" %s%s", command[j].cmd, command[j].parm);
      }
      printf("\n\n");
      return 0;
   }
   else if ( strcmp(argv[2], "functions") == 0 ) {
      printf("Assigned functions\nFunc Command\n");
      for ( codenum = 0; codenum < 64; codenum++ ) {
         for ( j = 0; j < (sizeof(command)/sizeof(struct command_st)); j++ ) {
            if ( command[j].codenum == codenum ) {
               printf(" %02d  %s\n", codenum, command[j].cmd);
               break;
            }
         }
      }
      printf("\nUnassigned functions:");
      for ( codenum = 0; codenum < 64; codenum++ ) {
         found = 0;
         for ( j = 0; j < (sizeof(command)/sizeof(struct command_st)); j++ ) {
            if ( command[j].codenum == codenum ) {
               found = 1;
               break;
            }
         }
         if ( !found )
            printf(" %02d", codenum);
      }
      printf("\n");
      return 0;
   }
            
   fprintf(stderr, "show options are:\n   %s\n   %s\n   %s\n   %s\n", msg01, msg02, msg03, msg04);

   return 1;
}

      
int config_device ( int argc, char *argv[] )
{
   char *msg01 = "Usage: %s config <device> <manf_name | all_manf | all_codes>\n";
   char *manf_name, *codestr, *dupestr;
   int  type, retcode;
   int  tokc;
   char *sp;
   char **tokv = NULL;

   if ( argc < 3 ) {
      fprintf(stderr, msg01, PROG_NAME);
      return 1;
   }
   if ( (type = identify_devtype(argv[2])) < 0 ) {
      fprintf(stderr, "Device '%s' not recognized - must be TV, VCR, DVD, CAB, SAT, or CD\n", argv[2]);
      return 1;
   }

   if ( argc < 4 ) {
      fprintf(stderr, msg01, PROG_NAME);
      return 1;
   }

   config_start_flag = 1;

   if ( strcmp(argv[3], "all_manf") == 0  ||
        strcmp(argv[3], "all_associated") == 0 ||
        strcmp(argv[3], "all_unassociated") == 0 ||
        strcmp(argv[3], "all_codes") == 0    ) {
      return ux_config(type, 1, argv + 3);

   }
   else if ( strtol(argv[3], &sp, 10) && strchr(" \t\n\r", *sp) ) {
      /* Numeric config codes entered on command line. */
      return ux_config(type, argc - 3, argv + 3);
   }
   else if ( find_manf(type, argv[3], &manf_name, &codestr) != 0 ) {
      /* Manf not found in list */ 
      return 1;
   }
   else {
      /* Test codes for specific manufacturer */
      dupestr = strdup(codestr);
      tokenize(dupestr, " ", &tokc, &tokv);
      printf("Testing device config codes for %s manufacturer %s\n",
         devlist[type].caplabel, manf_name);
      retcode = ux_config(type, tokc, tokv);
      free(tokv);
      free(dupestr);
      return retcode;
   }
   
   return 1;
}
 

void display_usage ( void )
{
   printf
    ("Usage: %s\n"
     "   version\n"
     "   init\n"
     "   show  manf <device>\n"
     "   show  codes <device> <manf_name>\n"
     "   show  commands\n"
     "   show  functions\n"
     "   config <device> <manf_name>\n"
     "   config <device> all_manf\n"
     "   config <device> all_codes\n"
     "   config <device> <code> [<code> [<code> ...]]]\n"
     "   <command> [<command> [<command> ...]]]\n", PROG_NAME
   );
   printf("\nwhere <device> is TV, VCR, DVD, CAB, SAT, or CD\n\n");

   return;
}

/*--------------------------------------------------------------------+
 | Function to execute the global command line.                       | 
 +--------------------------------------------------------------------*/
int exec_cmdline ( void )
{
   return system(cmdline);
}

   
